/*
 * kgraft_patch_bsc1124734
 *
 * Fix for CVE-2019-7221, bsc#1124734
 *
 *  Upstream commit:
 *  ecec76885bcf ("KVM: nVMX: unconditionally cancel preemption timer
 *                 in free_nested (CVE-2019-7221)")
 *
 *  SLE12(-SP1) commit:
 *  2cfc10fcb868c6205c09746afa7f037b3b9a3955
 *
 *  SLE12-SP2 commit:
 *  ff1adbba7a32004dab45ee00bb3290942c593e14
 *
 *  SLE12-SP3 commit:
 *  ff1adbba7a32004dab45ee00bb3290942c593e14
 *
 *  SLE12-SP4 commit:
 *  2b60717b20c2000bbe3cca8e324f91be217b0eeb
 *
 *  SLE15 commit:
 *  2b60717b20c2000bbe3cca8e324f91be217b0eeb
 *
 *
 *  Copyright (c) 2019 SUSE
 *  Author: Nicolai Stange <nstange@suse.de>
 *
 *  Based on the original Linux kernel code. Other copyrights apply.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#if IS_ENABLED(CONFIG_KVM_INTEL)

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/kvm_host.h>
#include <linux/list.h>
#include <asm/vmx.h>
#include <linux/hrtimer.h>
#include <linux/highmem.h>
#include <linux/gfp.h>
#include <linux/slab.h>
#include <asm/msr.h>
#include <asm/special_insns.h>
#include <asm/segment.h>
#include <asm/desc.h>
#include <asm/fpu/internal.h>
#include <asm/mce.h>
#include "kgr_patch_bsc1124734.h"
#include "kallsyms_relocs.h"

#if !IS_MODULE(CONFIG_KVM_INTEL)
#error "Live patch supports only CONFIG_KVM_INTEL=m"
#endif

#define KGR_PATCHED_MODULE "kvm_intel"


struct vcpu_vmx;
struct loaded_vmcs;
struct msr_data;

static bool (*kgr_enable_vpid);
static spinlock_t (*kgr_vmx_vpid_lock);
static DECLARE_BITMAP((*kgr_vmx_vpid_bitmap), VMX_NR_VPIDS);
static bool (*kgr_enable_shadow_vmcs);
static struct vmcs_config (*kgr_vmcs_config);
static bool (*kgr_enable_pml);
static struct desc_ptr __percpu (*kgr_host_gdt);
static bool (*kgr_cpu_has_load_ia32_efer);
static bool (*kgr_cpu_has_load_perf_global_ctrl);
static const u32 (*kgr_vmx_msr_index)[];
static u64 (*kgr_host_xss);

static void (*kgr_copy_shadow_to_vmcs12)(struct vcpu_vmx *vmx);
static void (*kgr_vmcs_clear_bits)(unsigned long field, u32 mask);
static void (*kgr_vmcs_writel)(unsigned long field, unsigned long value);
static void (*kgr_free_loaded_vmcs)(struct loaded_vmcs *loaded_vmcs);
static int (*kgr_nested_vmx_check_permission)(struct kvm_vcpu *vcpu);
static void (*kgr_skip_emulated_instruction)(struct kvm_vcpu *vcpu);
static void (*kgr_nested_vmx_succeed)(struct kvm_vcpu *vcpu);
static void (*kgr_vmx_load_vmcs01)(struct kvm_vcpu *vcpu);
static void (*kgr_nested_vmx_vmexit)(struct kvm_vcpu *vcpu, u32 exit_reason,
				     u32 exit_intr_info,
				     unsigned long exit_qualification);
static bool (*kgr_vmx_xsaves_supported)(void);
static void (*kgr_clear_atomic_switch_msr)(struct vcpu_vmx *vmx, unsigned msr);

static struct kmem_cache *(*kgr_kvm_vcpu_cache);

static void (*kgr_kvm_release_page_dirty)(struct page *page);
static int (*kgr_vcpu_load)(struct kvm_vcpu *vcpu);
static void (*kgr_vcpu_put)(struct kvm_vcpu *vcpu);
static void (*kgr_kvm_vcpu_uninit)(struct kvm_vcpu *vcpu);
static int (*kgr_kvm_set_msr_common)(struct kvm_vcpu *vcpu,
				     struct msr_data *msr_info);
static bool (*kgr_kvm_mpx_supported)(void);
static struct kvm_cpuid_entry2 *
(*kgr_kvm_find_cpuid_entry)(struct kvm_vcpu *vcpu,
			    u32 function, u32 index);
static void (*kgr_kvm_write_tsc)(struct kvm_vcpu *vcpu, struct msr_data *msr);
static bool (*kgr_kvm_mtrr_valid)(struct kvm_vcpu *vcpu, u32 msr, u64 data);
static int (*kgr_kvm_set_shared_msr)(unsigned slot, u64 value, u64 mask);

static struct kgr_kallsyms_reloc kgr_funcs[] = {
	{ "enable_vpid", (void *)&kgr_enable_vpid, "kvm_intel" },
	{ "vmx_vpid_lock", (void *)&kgr_vmx_vpid_lock, "kvm_intel" },
	{ "vmx_vpid_bitmap", (void *)&kgr_vmx_vpid_bitmap, "kvm_intel" },
	{ "enable_shadow_vmcs", (void *)&kgr_enable_shadow_vmcs, "kvm_intel" },
	{ "vmcs_config", (void *)&kgr_vmcs_config, "kvm_intel" },
	{ "enable_pml", (void *)&kgr_enable_pml, "kvm_intel" },
	{ "host_gdt", (void *)&kgr_host_gdt, "kvm_intel" },
	{ "cpu_has_load_ia32_efer", (void *)&kgr_cpu_has_load_ia32_efer,
	  "kvm_intel" },
	{ "cpu_has_load_perf_global_ctrl",
	  (void *)&kgr_cpu_has_load_perf_global_ctrl, "kvm_intel" },
	{ "vmx_msr_index", (void *)&kgr_vmx_msr_index, "kvm_intel" },
	{ "host_xss", (void *)&kgr_host_xss, "kvm_intel" },
	{ "copy_shadow_to_vmcs12", (void *)&kgr_copy_shadow_to_vmcs12,
	  "kvm_intel" },
	{ "vmcs_clear_bits", (void *)&kgr_vmcs_clear_bits, "kvm_intel" },
	{ "vmcs_writel", (void *)&kgr_vmcs_writel, "kvm_intel" },
	{ "free_loaded_vmcs", (void *)&kgr_free_loaded_vmcs, "kvm_intel" },
	{ "nested_vmx_check_permission",
	  (void *)&kgr_nested_vmx_check_permission, "kvm_intel" },
	{ "skip_emulated_instruction",
	  (void *)&kgr_skip_emulated_instruction, "kvm_intel" },
	{ "nested_vmx_succeed", (void *)&kgr_nested_vmx_succeed, "kvm_intel" },
	{ "vmx_load_vmcs01", (void *)&kgr_vmx_load_vmcs01, "kvm_intel" },
	{ "nested_vmx_vmexit", (void *)&kgr_nested_vmx_vmexit, "kvm_intel" },
	{ "vmx_xsaves_supported", (void *)&kgr_vmx_xsaves_supported,
	  "kvm_intel" },
	{ "clear_atomic_switch_msr", (void *)&kgr_clear_atomic_switch_msr,
	  "kvm_intel" },

	{ "kvm_vcpu_cache", (void *)&kgr_kvm_vcpu_cache, "kvm" },
	{ "kvm_release_page_dirty", (void *)&kgr_kvm_release_page_dirty,
	  "kvm" },
	{ "vcpu_load", (void *)&kgr_vcpu_load, "kvm" },
	{ "vcpu_put", (void *)&kgr_vcpu_put, "kvm" },
	{ "kvm_vcpu_uninit", (void *)&kgr_kvm_vcpu_uninit, "kvm" },
	{ "kvm_set_msr_common", (void *)&kgr_kvm_set_msr_common, "kvm" },
	{ "kvm_mpx_supported", (void *)&kgr_kvm_mpx_supported, "kvm" },
	{ "kvm_find_cpuid_entry", (void *)&kgr_kvm_find_cpuid_entry, "kvm" },
	{ "kvm_write_tsc", (void *)&kgr_kvm_write_tsc, "kvm" },
	{ "kvm_mtrr_valid", (void *)&kgr_kvm_mtrr_valid, "kvm" },
	{ "kvm_set_shared_msr", (void *)&kgr_kvm_set_shared_msr, "kvm" },
};


/* from arch/x86/kvm/kvm_cache_regs.h */
/* inlined */
static inline void kgr_leave_guest_mode(struct kvm_vcpu *vcpu)
{
	vcpu->arch.hflags &= ~HF_GUEST_MASK;
}

/* inlined */
static inline bool kgr_is_guest_mode(struct kvm_vcpu *vcpu)
{
	return vcpu->arch.hflags & HF_GUEST_MASK;
}


/* from arch/x86/kvm/x86.h */
/* inlined */
static inline int kgr_is_long_mode(struct kvm_vcpu *vcpu)
{
#ifdef CONFIG_X86_64
	return vcpu->arch.efer & EFER_LMA;
#else
	return 0;
#endif
}

/* inlined */
static inline u32 kgr_bit(int bitno)
{
	return 1 << (bitno & 31);
}


/* from arch/x86/kvm/cpuid.h */
/* inlined */
static inline bool kgr_guest_cpuid_has_mpx(struct kvm_vcpu *vcpu)
{
	struct kvm_cpuid_entry2 *best;

	best = kgr_kvm_find_cpuid_entry(vcpu, 7, 0);
	return best && (best->ebx & kgr_bit(X86_FEATURE_MPX));
}

/* inlined */
static inline bool kgr_guest_cpuid_has_rdtscp(struct kvm_vcpu *vcpu)
{
	struct kvm_cpuid_entry2 *best;

	best = kgr_kvm_find_cpuid_entry(vcpu, 0x80000001, 0);
	return best && (best->edx & kgr_bit(X86_FEATURE_RDTSCP));
}


/* from arch/x86/kvm/vmx.c */
#define KGR_NR_AUTOLOAD_MSRS 8

struct loaded_vmcs {
	struct vmcs *vmcs;
	int cpu;
	int launched;
	struct list_head loaded_vmcss_on_cpu_link;
};

struct shared_msr_entry {
	unsigned index;
	u64 data;
	u64 mask;
};

struct vmcs02_list {
	struct list_head list;
	gpa_t vmptr;
	struct loaded_vmcs vmcs02;
};

struct nested_vmx {
	/* Has the level1 guest done vmxon? */
	bool vmxon;
	gpa_t vmxon_ptr;

	/* The guest-physical address of the current VMCS L1 keeps for L2 */
	gpa_t current_vmptr;
	/* The host-usable pointer to the above */
	struct page *current_vmcs12_page;
	struct vmcs12 *current_vmcs12;
	struct vmcs *current_shadow_vmcs;
	/*
	 * Indicates if the shadow vmcs must be updated with the
	 * data hold by vmcs12
	 */
	bool sync_shadow_vmcs;

	/* vmcs02_list cache of VMCSs recently used to run L2 guests */
	struct list_head vmcs02_pool;
	int vmcs02_num;
	u64 vmcs01_tsc_offset;
	bool change_vmcs01_virtual_x2apic_mode;
	/* L2 must run next, and mustn't decide to exit to L1. */
	bool nested_run_pending;
	/*
	 * Guest pages referred to in vmcs02 with host-physical pointers, so
	 * we must keep them pinned while L2 runs.
	 */
	struct page *apic_access_page;
	struct page *virtual_apic_page;
	struct page *pi_desc_page;
	struct pi_desc *pi_desc;
	bool pi_pending;
	u16 posted_intr_nv;

	unsigned long *msr_bitmap;

	struct hrtimer preemption_timer;
	bool preemption_timer_expired;

	/* to migrate it to L2 if VM_ENTRY_LOAD_DEBUG_CONTROLS is off */
	u64 vmcs01_debugctl;

	u16 vpid02;
	u16 last_vpid;

	u32 nested_vmx_procbased_ctls_low;
	u32 nested_vmx_procbased_ctls_high;
	u32 nested_vmx_true_procbased_ctls_low;
	u32 nested_vmx_secondary_ctls_low;
	u32 nested_vmx_secondary_ctls_high;
	u32 nested_vmx_pinbased_ctls_low;
	u32 nested_vmx_pinbased_ctls_high;
	u32 nested_vmx_exit_ctls_low;
	u32 nested_vmx_exit_ctls_high;
	u32 nested_vmx_true_exit_ctls_low;
	u32 nested_vmx_entry_ctls_low;
	u32 nested_vmx_entry_ctls_high;
	u32 nested_vmx_true_entry_ctls_low;
	u32 nested_vmx_misc_low;
	u32 nested_vmx_misc_high;
	u32 nested_vmx_ept_caps;
	u32 nested_vmx_vpid_caps;
};

struct pi_desc {
	u32 pir[8];     /* Posted interrupt requested */
	union {
		struct {
				/* bit 256 - Outstanding Notification */
			u16	on	: 1,
				/* bit 257 - Suppress Notification */
				sn	: 1,
				/* bit 271:258 - Reserved */
				rsvd_1	: 14;
				/* bit 279:272 - Notification Vector */
			u8	nv;
				/* bit 287:280 - Reserved */
			u8	rsvd_2;
				/* bit 319:288 - Notification Destination */
			u32	ndst;
		};
		u64 control;
	};
	u32 rsvd[6];
} __aligned(64);

struct vcpu_vmx {
	struct kvm_vcpu       vcpu;
	unsigned long         host_rsp;
	u8                    fail;
	bool                  nmi_known_unmasked;
	u32                   exit_intr_info;
	u32                   idt_vectoring_info;
	ulong                 rflags;
	struct shared_msr_entry *guest_msrs;
	int                   nmsrs;
	int                   save_nmsrs;
	unsigned long	      host_idt_base;
#ifdef CONFIG_X86_64
	u64 		      msr_host_kernel_gs_base;
	u64 		      msr_guest_kernel_gs_base;
#endif
	u32 vm_entry_controls_shadow;
	u32 vm_exit_controls_shadow;
	/*
	 * loaded_vmcs points to the VMCS currently used in this vcpu. For a
	 * non-nested (L1) guest, it always points to vmcs01. For a nested
	 * guest (L2), it points to a different VMCS.
	 */
	struct loaded_vmcs    vmcs01;
	struct loaded_vmcs   *loaded_vmcs;
	bool                  __launched; /* temporary, used in vmx_vcpu_run */
	struct msr_autoload {
		unsigned nr;
		struct vmx_msr_entry guest[KGR_NR_AUTOLOAD_MSRS];
		struct vmx_msr_entry host[KGR_NR_AUTOLOAD_MSRS];
	} msr_autoload;
	struct {
		int           loaded;
		u16           fs_sel, gs_sel, ldt_sel;
#ifdef CONFIG_X86_64
		u16           ds_sel, es_sel;
#endif
		int           gs_ldt_reload_needed;
		int           fs_reload_needed;
		u64           msr_host_bndcfgs;
		unsigned long vmcs_host_cr4;	/* May not match real cr4 */
	} host_state;
	struct {
		int vm86_active;
		ulong save_rflags;
		struct kvm_segment segs[8];
	} rmode;
	struct {
		u32 bitmask; /* 4 bits per segment (1 bit per field) */
		struct kvm_save_segment {
			u16 selector;
			unsigned long base;
			u32 limit;
			u32 ar;
		} seg[8];
	} segment_cache;
	int vpid;
	bool emulation_required;

	/* Support for vnmi-less CPUs */
	int soft_vnmi_blocked;
	ktime_t entry_time;
	s64 vnmi_blocked_time;
	u32 exit_reason;

	/* Posted interrupt descriptor */
	struct pi_desc pi_desc;

	/* Support for a guest hypervisor (nested VMX) */
	struct nested_vmx nested;

	/* Dynamic PLE window. */
	int ple_window;
	bool ple_window_dirty;

	/* Support for PML */
#define KGR_PML_ENTITY_NUM		512
	struct page *pml_pg;

	u64 current_tsc_ratio;

	u32 guest_pkru;
	u32 host_pkru;

	/*
	 * Only bits masked by msr_ia32_feature_control_valid_bits can be set in
	 * msr_ia32_feature_control. FEATURE_CONTROL_LOCKED is always included
	 * in msr_ia32_feature_control_valid_bits.
	 */
	u64 msr_ia32_feature_control;
	u64 msr_ia32_feature_control_valid_bits;

	u64 spec_ctrl;
};

/* inlined */
static inline struct vcpu_vmx *kgr_to_vmx(struct kvm_vcpu *vcpu)
{
	return container_of(vcpu, struct vcpu_vmx, vcpu);
}

/* inlined */
static void kgr_nested_release_page(struct page *page)
{
	kgr_kvm_release_page_dirty(page);
}

struct vmcs_config {
	int size;
	int order;
	u32 revision_id;
	u32 pin_based_exec_ctrl;
	u32 cpu_based_exec_ctrl;
	u32 cpu_based_2nd_exec_ctrl;
	u32 vmexit_ctrl;
	u32 vmentry_ctrl;
};

/* inlined */
static int kgr__find_msr_index(struct vcpu_vmx *vmx, u32 msr)
{
	int i;

	for (i = 0; i < vmx->nmsrs; ++i)
		if ((*kgr_vmx_msr_index)[vmx->guest_msrs[i].index] == msr)
			return i;
	return -1;
}

/* inlined */
static struct shared_msr_entry *kgr_find_msr_entry(struct vcpu_vmx *vmx, u32 msr)
{
	int i;

	i = kgr__find_msr_index(vmx, msr);
	if (i >= 0)
		return &vmx->guest_msrs[i];
	return NULL;
}

/* inlined */
static void kgr_vmcs_write32(unsigned long field, u32 value)
{
	kgr_vmcs_writel(field, value);
}

/* inlined */
static void kgr_vmcs_write64(unsigned long field, u64 value)
{
	kgr_vmcs_writel(field, value);
#ifndef CONFIG_X86_64
	asm volatile ("");
	kgr_vmcs_writel(field+1, value >> 32);
#endif
}

/* inlined */
static inline void kgr_vm_entry_controls_init(struct vcpu_vmx *vmx, u32 val)
{
	kgr_vmcs_write32(VM_ENTRY_CONTROLS, val);
	vmx->vm_entry_controls_shadow = val;
}

/* inlined */
static inline void kgr_vm_entry_controls_set(struct vcpu_vmx *vmx, u32 val)
{
	if (vmx->vm_entry_controls_shadow != val)
		kgr_vm_entry_controls_init(vmx, val);
}

/* inined */
static inline u32 kgr_vm_entry_controls_get(struct vcpu_vmx *vmx)
{
	return vmx->vm_entry_controls_shadow;
}

/* inlined */
static inline void kgr_vm_entry_controls_setbit(struct vcpu_vmx *vmx, u32 val)
{
	kgr_vm_entry_controls_set(vmx, kgr_vm_entry_controls_get(vmx) | val);
}

/* inlined */
static inline void kgr_vm_exit_controls_init(struct vcpu_vmx *vmx, u32 val)
{
	kgr_vmcs_write32(VM_EXIT_CONTROLS, val);
	vmx->vm_exit_controls_shadow = val;
}

/* inlined */
static inline void kgr_vm_exit_controls_set(struct vcpu_vmx *vmx, u32 val)
{
	if (vmx->vm_exit_controls_shadow != val)
		kgr_vm_exit_controls_init(vmx, val);
}

/* inlined */
static inline u32 kgr_vm_exit_controls_get(struct vcpu_vmx *vmx)
{
	return vmx->vm_exit_controls_shadow;
}

/* inlined */
static inline void kgr_vm_exit_controls_setbit(struct vcpu_vmx *vmx, u32 val)
{
	kgr_vm_exit_controls_set(vmx, kgr_vm_exit_controls_get(vmx) | val);
}

/* inlined */
static void kgr_vmx_segment_cache_clear(struct vcpu_vmx *vmx)
{
	vmx->segment_cache.bitmask = 0;
}

/* inlined */
static void kgr_add_atomic_switch_msr_special(struct vcpu_vmx *vmx,
		unsigned long entry, unsigned long exit,
		unsigned long guest_val_vmcs, unsigned long host_val_vmcs,
		u64 guest_val, u64 host_val)
{
	kgr_vmcs_write64(guest_val_vmcs, guest_val);
	kgr_vmcs_write64(host_val_vmcs, host_val);
	kgr_vm_entry_controls_setbit(vmx, entry);
	kgr_vm_exit_controls_setbit(vmx, exit);
}

/* inlined */
static void kgr_add_atomic_switch_msr(struct vcpu_vmx *vmx, unsigned msr,
				      u64 guest_val, u64 host_val)
{
	unsigned i;
	struct msr_autoload *m = &vmx->msr_autoload;

	switch (msr) {
	case MSR_EFER:
		if ((*kgr_cpu_has_load_ia32_efer)) {
			kgr_add_atomic_switch_msr_special(vmx,
					VM_ENTRY_LOAD_IA32_EFER,
					VM_EXIT_LOAD_IA32_EFER,
					GUEST_IA32_EFER,
					HOST_IA32_EFER,
					guest_val, host_val);
			return;
		}
		break;
	case MSR_CORE_PERF_GLOBAL_CTRL:
		if ((*kgr_cpu_has_load_perf_global_ctrl)) {
			kgr_add_atomic_switch_msr_special(vmx,
					VM_ENTRY_LOAD_IA32_PERF_GLOBAL_CTRL,
					VM_EXIT_LOAD_IA32_PERF_GLOBAL_CTRL,
					GUEST_IA32_PERF_GLOBAL_CTRL,
					HOST_IA32_PERF_GLOBAL_CTRL,
					guest_val, host_val);
			return;
		}
		break;
	case MSR_IA32_PEBS_ENABLE:
		/* PEBS needs a quiescent period after being disabled (to write
		 * a record).  Disabling PEBS through VMX MSR swapping doesn't
		 * provide that period, so a CPU could write host's record into
		 * guest's memory.
		 */
		wrmsrl(MSR_IA32_PEBS_ENABLE, 0);
	}

	for (i = 0; i < m->nr; ++i)
		if (m->guest[i].index == msr)
			break;

	if (i == KGR_NR_AUTOLOAD_MSRS) {
		printk_once(KERN_WARNING "Not enough msr switch entries. "
				"Can't add msr %x\n", msr);
		return;
	} else if (i == m->nr) {
		++m->nr;
		kgr_vmcs_write32(VM_ENTRY_MSR_LOAD_COUNT, m->nr);
		kgr_vmcs_write32(VM_EXIT_MSR_LOAD_COUNT, m->nr);
	}

	m->guest[i].index = msr;
	m->guest[i].value = guest_val;
	m->host[i].index = msr;
	m->host[i].value = host_val;
}

/* inlined */
static void kgr_reload_tss(void)
{
	/*
	 * VT restores TR but not its size.  Useless.
	 */
	struct desc_ptr *gdt = this_cpu_ptr(&(*kgr_host_gdt));
	struct desc_struct *descs;

	descs = (void *)gdt->address;
	descs[GDT_ENTRY_TSS].type = 9; /* available TSS */
	load_TR_desc();
}

/* inlined */
static void kgr__vmx_load_host_state(struct vcpu_vmx *vmx)
{
	if (!vmx->host_state.loaded)
		return;

	++vmx->vcpu.stat.host_state_reload;
	vmx->host_state.loaded = 0;
#ifdef CONFIG_X86_64
	if (kgr_is_long_mode(&vmx->vcpu))
		rdmsrl(MSR_KERNEL_GS_BASE, vmx->msr_guest_kernel_gs_base);
#endif
	if (vmx->host_state.gs_ldt_reload_needed) {
		kvm_load_ldt(vmx->host_state.ldt_sel);
#ifdef CONFIG_X86_64
		load_gs_index(vmx->host_state.gs_sel);
#else
		loadsegment(gs, vmx->host_state.gs_sel);
#endif
	}
	if (vmx->host_state.fs_reload_needed)
		loadsegment(fs, vmx->host_state.fs_sel);
#ifdef CONFIG_X86_64
	if (unlikely(vmx->host_state.ds_sel | vmx->host_state.es_sel)) {
		loadsegment(ds, vmx->host_state.ds_sel);
		loadsegment(es, vmx->host_state.es_sel);
	}
#endif
	kgr_reload_tss();
#ifdef CONFIG_X86_64
	wrmsrl(MSR_KERNEL_GS_BASE, vmx->msr_host_kernel_gs_base);
#endif
	if (vmx->host_state.msr_host_bndcfgs)
		wrmsrl(MSR_IA32_BNDCFGS, vmx->host_state.msr_host_bndcfgs);
	/*
	 * If the FPU is not active (through the host task or
	 * the guest vcpu), then restore the cr0.TS bit.
	 */
	if (!fpregs_active() && !vmx->vcpu.guest_fpu_loaded)
		stts();
	load_gdt(this_cpu_ptr(&(*kgr_host_gdt)));
}

/* inlined */
static void kgr_vmx_load_host_state(struct vcpu_vmx *vmx)
{
	preempt_disable();
	kgr__vmx_load_host_state(vmx);
	preempt_enable();
}

/* inlined */
static inline bool kgr_vmx_feature_control_msr_valid(struct kvm_vcpu *vcpu,
						     uint64_t val)
{
	uint64_t valid_bits = kgr_to_vmx(vcpu)->msr_ia32_feature_control_valid_bits;

	return !(val & ~valid_bits);
}

/* inlined */
static void kgr_free_vmcs(struct vmcs *vmcs)
{
	free_pages((unsigned long)vmcs, (*kgr_vmcs_config).order);
}

/* inlined */
static void kgr_free_vpid(int vpid)
{
	if (!(*kgr_enable_vpid) || vpid == 0)
		return;
	spin_lock(&(*kgr_vmx_vpid_lock));
	__clear_bit(vpid, (*kgr_vmx_vpid_bitmap));
	spin_unlock(&(*kgr_vmx_vpid_lock));
}

/* inlined */
static void kgr_nested_free_all_saved_vmcss(struct vcpu_vmx *vmx)
{
	struct vmcs02_list *item, *n;

	WARN_ON(vmx->loaded_vmcs != &vmx->vmcs01);
	list_for_each_entry_safe(item, n, &vmx->nested.vmcs02_pool, list) {
		/*
		 * Something will leak if the above WARN triggers.  Better than
		 * a use-after-free.
		 */
		if (vmx->loaded_vmcs == &item->vmcs02)
			continue;

		kgr_free_loaded_vmcs(&item->vmcs02);
		list_del(&item->list);
		kfree(item);
		vmx->nested.vmcs02_num--;
	}
}

/* inlined */
static inline void kgr_nested_release_vmcs12(struct vcpu_vmx *vmx)
{
	if (vmx->nested.current_vmptr == -1ull)
		return;

	/* current_vmptr and current_vmcs12 are always set/reset together */
	if (WARN_ON(vmx->nested.current_vmcs12 == NULL))
		return;

	if ((*kgr_enable_shadow_vmcs)) {
		/* copy to memory all shadowed fields in case
		   they were modified */
		kgr_copy_shadow_to_vmcs12(vmx);
		vmx->nested.sync_shadow_vmcs = false;
		kgr_vmcs_clear_bits(SECONDARY_VM_EXEC_CONTROL,
				    SECONDARY_EXEC_SHADOW_VMCS);
		kgr_vmcs_write64(VMCS_LINK_POINTER, -1ull);
	}
	vmx->nested.posted_intr_nv = -1;
	kunmap(vmx->nested.current_vmcs12_page);
	kgr_nested_release_page(vmx->nested.current_vmcs12_page);
	vmx->nested.current_vmptr = -1ull;
	vmx->nested.current_vmcs12 = NULL;
}

/* inlined */
static void kgr_vmx_destroy_pml_buffer(struct vcpu_vmx *vmx)
{
	if (vmx->pml_pg) {
		__free_page(vmx->pml_pg);
		vmx->pml_pg = NULL;
	}
}



/* patched, inlined */
static void kgr_free_nested(struct vcpu_vmx *vmx)
{
	if (!vmx->nested.vmxon)
		return;

	/*
	 * Fix CVE-2019-7221
	 *  +1 line
	 */
	hrtimer_cancel(&vmx->nested.preemption_timer);
	vmx->nested.vmxon = false;
	kgr_free_vpid(vmx->nested.vpid02);
	kgr_nested_release_vmcs12(vmx);
	if (vmx->nested.msr_bitmap) {
		free_page((unsigned long)vmx->nested.msr_bitmap);
		vmx->nested.msr_bitmap = NULL;
	}
	if ((*kgr_enable_shadow_vmcs))
		kgr_free_vmcs(vmx->nested.current_shadow_vmcs);
	/* Unpin physical memory we referred to in current vmcs02 */
	if (vmx->nested.apic_access_page) {
		kgr_nested_release_page(vmx->nested.apic_access_page);
		vmx->nested.apic_access_page = NULL;
	}
	if (vmx->nested.virtual_apic_page) {
		kgr_nested_release_page(vmx->nested.virtual_apic_page);
		vmx->nested.virtual_apic_page = NULL;
	}
	if (vmx->nested.pi_desc_page) {
		kunmap(vmx->nested.pi_desc_page);
		kgr_nested_release_page(vmx->nested.pi_desc_page);
		vmx->nested.pi_desc_page = NULL;
		vmx->nested.pi_desc = NULL;
	}

	kgr_nested_free_all_saved_vmcss(vmx);
}

/* patched, calls inlined free_nested() */
int kgr_handle_vmoff(struct kvm_vcpu *vcpu)
{
	if (!kgr_nested_vmx_check_permission(vcpu))
		return 1;
	kgr_free_nested(kgr_to_vmx(vcpu));
	kgr_skip_emulated_instruction(vcpu);
	kgr_nested_vmx_succeed(vcpu);
	return 1;
}

/* patched, inlined, calls inlined free_nested() */
static void kgr_vmx_free_vcpu_nested(struct kvm_vcpu *vcpu)
{
       struct vcpu_vmx *vmx = kgr_to_vmx(vcpu);
       int r;

       r = kgr_vcpu_load(vcpu);
       BUG_ON(r);
       kgr_vmx_load_vmcs01(vcpu);
       kgr_free_nested(vmx);
       kgr_vcpu_put(vcpu);
}

/* patched, calls inlined vmx_free_vcpu_nested() */
void kgr_vmx_free_vcpu(struct kvm_vcpu *vcpu)
{
	struct vcpu_vmx *vmx = kgr_to_vmx(vcpu);

	if ((*kgr_enable_pml))
		kgr_vmx_destroy_pml_buffer(vmx);
	kgr_free_vpid(vmx->vpid);
	kgr_leave_guest_mode(vcpu);
	kgr_vmx_free_vcpu_nested(vcpu);
	kgr_free_loaded_vmcs(vmx->loaded_vmcs);
	kfree(vmx->guest_msrs);
	kgr_kvm_vcpu_uninit(vcpu);
	kmem_cache_free((*kgr_kvm_vcpu_cache), vmx);
}

/* patched, inlined, calls inlined free_nested() */
static void kgr_vmx_leave_nested(struct kvm_vcpu *vcpu)
{
	if (kgr_is_guest_mode(vcpu)) {
		kgr_to_vmx(vcpu)->nested.nested_run_pending = 0;
		kgr_nested_vmx_vmexit(vcpu, -1, 0, 0);
	}
	kgr_free_nested(kgr_to_vmx(vcpu));
}

/* patched, calls inlined vmx_leave_nested() */
int kgr_vmx_set_msr(struct kvm_vcpu *vcpu, struct msr_data *msr_info)
{
	struct vcpu_vmx *vmx = kgr_to_vmx(vcpu);
	struct shared_msr_entry *msr;
	int ret = 0;
	u32 msr_index = msr_info->index;
	u64 data = msr_info->data;

	switch (msr_index) {
	case MSR_EFER:
		ret = kgr_kvm_set_msr_common(vcpu, msr_info);
		break;
#ifdef CONFIG_X86_64
	case MSR_FS_BASE:
		kgr_vmx_segment_cache_clear(vmx);
		kgr_vmcs_writel(GUEST_FS_BASE, data);
		break;
	case MSR_GS_BASE:
		kgr_vmx_segment_cache_clear(vmx);
		kgr_vmcs_writel(GUEST_GS_BASE, data);
		break;
	case MSR_KERNEL_GS_BASE:
		kgr_vmx_load_host_state(vmx);
		vmx->msr_guest_kernel_gs_base = data;
		break;
#endif
	case MSR_IA32_SYSENTER_CS:
		kgr_vmcs_write32(GUEST_SYSENTER_CS, data);
		break;
	case MSR_IA32_SYSENTER_EIP:
		kgr_vmcs_writel(GUEST_SYSENTER_EIP, data);
		break;
	case MSR_IA32_SYSENTER_ESP:
		kgr_vmcs_writel(GUEST_SYSENTER_ESP, data);
		break;
	case MSR_IA32_BNDCFGS:
		if (!kgr_kvm_mpx_supported() ||
		    (!msr_info->host_initiated && !kgr_guest_cpuid_has_mpx(vcpu)))
			return 1;
		if (is_noncanonical_address(data & PAGE_MASK) ||
		    (data & MSR_IA32_BNDCFGS_RSVD))
			return 1;
		kgr_vmcs_write64(GUEST_BNDCFGS, data);
		break;
	case MSR_IA32_TSC:
		kgr_kvm_write_tsc(vcpu, msr_info);
		break;
	case MSR_IA32_SPEC_CTRL:
		vmx->spec_ctrl = msr_info->data;
		break;
	case MSR_IA32_CR_PAT:
		if ((*kgr_vmcs_config).vmentry_ctrl & VM_ENTRY_LOAD_IA32_PAT) {
			if (!kgr_kvm_mtrr_valid(vcpu, MSR_IA32_CR_PAT, data))
				return 1;
			kgr_vmcs_write64(GUEST_IA32_PAT, data);
			vcpu->arch.pat = data;
			break;
		}
		ret = kgr_kvm_set_msr_common(vcpu, msr_info);
		break;
	case MSR_IA32_TSC_ADJUST:
		ret = kgr_kvm_set_msr_common(vcpu, msr_info);
		break;
	case MSR_IA32_MCG_EXT_CTL:
		if ((!msr_info->host_initiated &&
		     !(kgr_to_vmx(vcpu)->msr_ia32_feature_control &
		       FEATURE_CONTROL_LMCE)) ||
		    (data & ~MCG_EXT_CTL_LMCE_EN))
			return 1;
		vcpu->arch.mcg_ext_ctl = data;
		break;
	case MSR_IA32_FEATURE_CONTROL:
		if (!kgr_vmx_feature_control_msr_valid(vcpu, data) ||
		    (kgr_to_vmx(vcpu)->msr_ia32_feature_control &
		     FEATURE_CONTROL_LOCKED && !msr_info->host_initiated))
			return 1;
		vmx->msr_ia32_feature_control = data;
		if (msr_info->host_initiated && data == 0)
			kgr_vmx_leave_nested(vcpu);
		break;
	case MSR_IA32_VMX_BASIC ... MSR_IA32_VMX_VMFUNC:
		return 1; /* they are read-only */
	case MSR_IA32_XSS:
		if (!kgr_vmx_xsaves_supported())
			return 1;
		/*
		 * The only supported bit as of Skylake is bit 8, but
		 * it is not supported on KVM.
		 */
		if (data != 0)
			return 1;
		vcpu->arch.ia32_xss = data;
		if (vcpu->arch.ia32_xss != (*kgr_host_xss))
			kgr_add_atomic_switch_msr(vmx, MSR_IA32_XSS,
				vcpu->arch.ia32_xss, (*kgr_host_xss));
		else
			kgr_clear_atomic_switch_msr(vmx, MSR_IA32_XSS);
		break;
	case MSR_TSC_AUX:
		if (!kgr_guest_cpuid_has_rdtscp(vcpu) && !msr_info->host_initiated)
			return 1;
		/* Check reserved bit, higher 32 bits should be zero */
		if ((data >> 32) != 0)
			return 1;
		/* Otherwise falls through */
	default:
		msr = kgr_find_msr_entry(vmx, msr_index);
		if (msr) {
			u64 old_msr_data = msr->data;
			msr->data = data;
			if (msr - vmx->guest_msrs < vmx->save_nmsrs) {
				preempt_disable();
				ret = kgr_kvm_set_shared_msr(msr->index, msr->data,
							     msr->mask);
				preempt_enable();
				if (ret)
					msr->data = old_msr_data;
			}
			break;
		}
		ret = kgr_kvm_set_msr_common(vcpu, msr_info);
	}

	return ret;
}



static int kgr_patch_bsc1124734_module_notify(struct notifier_block *nb,
					      unsigned long action, void *data)
{
	struct module *mod = data;
	int ret;

	if (action != MODULE_STATE_COMING || strcmp(mod->name, KGR_PATCHED_MODULE))
		return 0;

	ret = __kgr_resolve_kallsyms_relocs(kgr_funcs, ARRAY_SIZE(kgr_funcs));
	WARN(ret, "kgraft-patch: delayed kallsyms lookup failed. System is broken and can crash.\n");

	return ret;
}

static struct notifier_block kgr_patch_bsc1124734_module_nb = {
	.notifier_call = kgr_patch_bsc1124734_module_notify,
	.priority = INT_MIN+1,
};

int kgr_patch_bsc1124734_init(void)
{
	int ret;

	mutex_lock(&module_mutex);
	if (find_module(KGR_PATCHED_MODULE)) {
		ret = __kgr_resolve_kallsyms_relocs(kgr_funcs,
						    ARRAY_SIZE(kgr_funcs));
		if (ret)
			goto out;
	}

	ret = register_module_notifier(&kgr_patch_bsc1124734_module_nb);
out:
	mutex_unlock(&module_mutex);
	return ret;
}

void kgr_patch_bsc1124734_cleanup(void)
{
	unregister_module_notifier(&kgr_patch_bsc1124734_module_nb);
}

#endif /* IS_ENABLED(CONFIG_KVM_INTEL) */
